/*
 * {{{ header & license
 * Copyright (c) 2006 Wisconsin Court System
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * }}}
 */
package org.xhtmlrenderer.swing;

import java.awt.Point;

import javax.swing.JComponent;

import org.xhtmlrenderer.extend.ReplacedElement;
import org.xhtmlrenderer.layout.LayoutContext;

public class SwingReplacedElement implements ReplacedElement {
    private JComponent _component;
    
    public SwingReplacedElement(JComponent component) {
        _component = component;
    }
    
    public JComponent getJComponent() {
        return _component;
    }
    
    public int getIntrinsicHeight() {
        return _component.getBounds().height;
    }
    
    public int getIntrinsicWidth() {
        return _component.getBounds().width;
    }
    
    public void setLocation(int x, int y) {
        _component.setLocation(x, y);
    }
    
    public Point getLocation() {
        return _component.getLocation();
    }
    
    public void detach(LayoutContext c) {
        if (c.isInteractive()) {
            c.getCanvas().remove(getJComponent());
        }
    }
    
    public boolean isRequiresInteractivePaint() {
        return false;
    }

	public int getBaseline() {
		return 0;
	}

	public boolean hasBaseline() {
		return false;
	}
}
